/*
 * grader-io - irqflags.h
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 * based on Linux kernel on GPLv2 license
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifndef IRQFLAGS_H_INCLUDED
#define IRQFLAGS_H_INCLUDED

#ifdef HAVE_UNISTD_H

extern int get_signals(void);
extern int set_signals(int enable);
extern void block_signals(void);
extern void unblock_signals(void);

static inline unsigned long arch_local_save_flags(void)
{
	return get_signals();
}

#define local_irq_restore local_irq_restore
static inline void local_irq_restore(unsigned long flags)
{
	set_signals(flags);
}

#define local_irq_enable local_irq_enable
static inline void local_irq_enable(void)
{
	unblock_signals();
}

#define local_irq_disable local_irq_disable
static inline void local_irq_disable(void)
{
	block_signals();
}

#else
#define IRQMASK_I_BIT	1

#define arch_local_irq_save arch_local_irq_save
static inline unsigned long arch_local_irq_save(void)
{
	unsigned long flags;

	asm volatile(
		"	mrs	%0, primask	@ local_irq_save\n"
		"	cpsid	i"
		: "=r" (flags) : : "memory", "cc");
	return flags;
}

#define local_irq_enable local_irq_enable
static inline void local_irq_enable(void)
{
	asm volatile(
		"	cpsie i			@ local_irq_enable"
		:
		:
		: "memory", "cc");
}

#define local_irq_disable local_irq_disable
static inline void local_irq_disable(void)
{
	asm volatile(
		"	cpsid i			@ local_irq_disable"
		:
		:
		: "memory", "cc");
}

/*
 * Save the current interrupt enable state.
 */
#define local_save_flags local_save_flags
static inline unsigned long local_save_flags(void)
{
	unsigned long flags;

	asm volatile(
		"	mrs	%0, primask	@ local_save_flags"
		: "=r" (flags) : : "memory", "cc");
	return flags;
}

/*
 * restore saved IRQ & FIQ state
 */
#define local_irq_restore local_irq_restore
static inline void local_irq_restore(unsigned long flags)
{
	asm volatile(
		"	msr	primask, %0	@ local_irq_restore"
		:
		: "r" (flags)
		: "memory", "cc");
}

#define irqs_disabled_flags irqs_disabled_flags
static inline int irqs_disabled_flags(unsigned long flags)
{
	return flags & IRQMASK_I_BIT;
}
#endif

/* get status and disable interrupts */
#ifndef arch_local_irq_save
static inline unsigned long arch_local_irq_save(void)
{
	unsigned long flags;

	flags = arch_local_save_flags();
	local_irq_disable();
	return flags;
}
#endif

#define local_irq_save(flags) \
	do { \
		flags = arch_local_irq_save(); \
	} while (0)

#endif
