/*
 * crc14_test.c
 *
 * CRC-14 test
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <crc14.h>
#include <stdio.h>
#include <stdlib.h>

#define CRC_INIT	0x3fff
#define CRC_CHECK	0x3c90

#define HD		6
#define ERRORS		(HD - 1)
#define BITS		70
#define BURST_MAX	70

#define error(fmt...) fprintf(stderr, "error:" fmt)

static void change_bit(const unsigned char *ori, unsigned int bit,
		unsigned char *msg)
{
	unsigned int byte = bit / 7;
	unsigned int bit_in_byte = bit % 7;
	unsigned int mask = 1 << bit_in_byte;

	if (ori[byte] & mask)
		msg[byte] &= ~mask;
	else
		msg[byte] |= mask;
}

int main(void)
{
	unsigned char msg_ori[10];
	unsigned char msg[10];
	unsigned int bit[ERRORS];
	unsigned int i;
	unsigned long tested = 0;
	unsigned int crc;
	unsigned int start;
	unsigned int burst;

	for (i = 0; i < ERRORS; i++)
		bit[i] = 0;

	memset(msg_ori, 0, sizeof(msg_ori));
	crc = crc14(CRC_INIT, msg_ori, sizeof(msg_ori) - 2);
	crc = ~crc;
	msg_ori[sizeof(msg_ori) - 2] = (crc >> 0) & 0x7f;
	msg_ori[sizeof(msg_ori) - 1] = (crc >> 7) & 0x7f;

	crc = crc14(CRC_INIT, msg_ori, sizeof(msg_ori));
	if (crc != CRC_CHECK) {
		error("invalid initial checksum: %04x\n", crc);
		exit(EXIT_FAILURE);
	}

	for (burst = 1; burst <= BURST_MAX; burst++) {
		for (start = 0; start <= BITS - burst; start++) {
			memcpy(msg, msg_ori, sizeof(msg));
			for (i = 0; i < burst; i++)
				change_bit(msg_ori, start + i, msg);

			tested++;
			if (crc14(CRC_INIT, msg, sizeof(msg)) == CRC_CHECK) {
				error("burst error %d-%d not detected\n",
						start, start + burst - 1);
				fprintf(stderr, "message:");
				for (i = 0; i < sizeof(msg); i++)
					fprintf(stderr, " 0x%02x", msg[i]);
				fprintf(stderr, "\n");
				exit(EXIT_FAILURE);
			}
		}
	}

	while (1) {
		memcpy(msg, msg_ori, sizeof(msg));
		tested++;
		for (i = 0; i < ERRORS; i++)
			change_bit(msg_ori, bit[i], msg);

		if (crc14(CRC_INIT, msg, sizeof(msg)) == CRC_CHECK) {
			fprintf(stderr, "error: error with bits");
			for (i = 0; i < ERRORS; i++)
				fprintf(stderr, " %d", bit[i]);
			fprintf(stderr, " not detected\n");
			fprintf(stderr, "message:");
			for (i = 0; i < sizeof(msg); i++)
				fprintf(stderr, " 0x%02x", msg[i]);
			fprintf(stderr, "\n");
			exit(EXIT_FAILURE);
		}

		for (i = 0; ;) {
			if (bit[i] < BITS - 1) {
				bit[i]++;
				break;
			}
			bit[i++] = 0;
			if (i >= ERRORS) {
				fprintf(stderr, "%ld cases\n", tested);
				exit(EXIT_SUCCESS);
			}
		}
	}
}
