/*
 * grader-rt - posix-encoder.c
 *
 * Copyright (C) 2017  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <timer.h>
#include <ctype.h>
#include <tty.h>
#include <stdio.h>

#define VELOCITY_MAX	65555

static struct tty_struct *encoder_tty;
static unsigned int velocity = 1000;

unsigned long encoder_get_pos(void)
{
	static unsigned long long phase;
	static unsigned long pos;
	static int not_first;
	static unsigned long prev_t;
	unsigned long t;

	if (!not_first) {
		prev_t = gettime();
		not_first = 1;
	}

	t = gettime();

	phase += (unsigned long long) velocity * (t - prev_t);
	prev_t = t;

	pos += phase / GETTIME_FREQUENCY;
	phase %= GETTIME_FREQUENCY;
	return pos;
}

static void posix_encoder_receive_buf(struct tty_struct *tty,
		const unsigned char *cp, char *fp, int count)
{
	static unsigned int x;

	if (!count)
		return;

	do {
		unsigned char c;

		if (fp && *fp++) {
			cp++;
			continue;
		}

		c = *cp++;
		if (isdigit(c)) {
			x = x * 10 + c - '0';
		} else if (c == '\n') {
			if (x > VELOCITY_MAX)
				x = VELOCITY_MAX;
			velocity = x;
			x = 0;
		}
		count--;
	} while (count > 0);
}

static const struct tty_ldisc_ops posix_encoder_ldisc_ops = {
	.receive_buf		= posix_encoder_receive_buf,
};

int encoder_init(void)
{
	encoder_tty = serial_init(1);
	tty_set_ldisc(encoder_tty, &posix_encoder_ldisc_ops, (void *)1);
	return 0;
}
