/*
 * grader-io - timer.c
 *
 * NXP timer driver
 *
 * Copyright (C) 2016  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <asm_io.h>
#include <lpc176x.h>
#include <timer.h>

#define SYSTICK_BASE		0xe000e000
#define SYST_CSR		0x010
#define SYST_CSR_ENABLE		0x01
#define SYST_CSR_TICKINT	0x02
#define SYST_CSR_CLKSOURCE	0x04
#define SYST_RVR		0x014
#define SYST_CVR		0x018
#define SYST_CALIB		0x01c

static u32 systick_read(unsigned int reg)
{
	return readl((u8 *) SYSTICK_BASE + reg);
}

static void systick_write(u32 x, unsigned int reg)
{
	writel(x, (u8 *) SYSTICK_BASE + reg);
}

unsigned long gettime(void)
{
	static unsigned long et;
	static unsigned long oldt;
	unsigned long t;

	t = -(systick_read(SYST_CVR) << 8);

	et &= ~0x00ffffff;
	et |= (t >> 8) & 0x00ffffff;
	if (t < oldt)
		et += 0x01000000;
	oldt = t;
	return et;
}

void timer_init(void)
{
	systick_write(0xffffff, SYST_RVR);
	systick_write(SYST_CSR_ENABLE | SYST_CSR_CLKSOURCE, SYST_CSR);
}
