/*
 * grader - tty-pipe.c
 *
 * Copyright (C) 2017  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <kernel.h>
#include <tty.h>
#include <tty-pipe.h>

#define PIPE_MAX	4

#define PIPE_BUF	32

struct tty_pipe {
	struct tty_struct tty;

	unsigned char buf[PIPE_BUF];
	unsigned int head;
	unsigned int tail;
};

static struct tty_pipe tty_pipe_table[PIPE_MAX];

static int tty_pipe_tty_write(struct tty_struct *tty, const unsigned char *buf,
		int count)
{
	struct tty_pipe *p = container_of(tty, struct tty_pipe, tty);
	unsigned char *pbuf = p->buf;
	int ret = 0;
	unsigned int tail;
	unsigned int head;
	unsigned int next;

	tail = p->tail;
	head = p->head;
	while (count > 0) {
		next = (head + 1) & (PIPE_BUF - 1);
		if (next == tail)
			break;
		pbuf[head] = *buf++;
		head = next;
		count--;
		ret++;
	}
	p->head = head;
	return ret;
}

static const struct tty_operations tty_pipe_tty_operations = {
	.write		= tty_pipe_tty_write,
};

int tty_pipe_getchar(struct tty_struct *tty)
{
	struct tty_pipe *p = container_of(tty, struct tty_pipe, tty);
	int ret;

	if (p->head == p->tail)
		return -1;
	ret = p->buf[p->tail];
	p->tail = (p->tail + 1) & (PIPE_BUF - 1);
	return ret;
}

struct tty_struct *tty_pipe(void)
{
	struct tty_pipe *p;

	for (p = tty_pipe_table; p < tty_pipe_table + PIPE_MAX; p++) {
		if (!p->tty.ops) {
			p->tty.ops = &tty_pipe_tty_operations;
			return &p->tty;
		}
	}
	return NULL;
}
